@@ -104,6 +104,7 @@ const char* efi_bootname[3] = { "boot", "grub", "mm" };
 const char* efi_archname[ARCH_MAX] = { "", "ia32", "x64", "arm", "aa64", "ia64", "riscv64", "loongarch64", "ebc" };
 static const char* sources_str = "/sources";
 static const char* wininst_name[] = { "install.wim", "install.esd", "install.swm" };
+_STATIC_ASSERT(ARRAYSIZE(wininst_name) < 4);	// Must fit as 4 bit position flag
 // We only support GRUB/BIOS (x86) that uses a standard config dir (/boot/grub/i386-pc/)
 // If the disc was mastered properly, GRUB/EFI will take care of itself
 static const char* grub_dirname[] = { "/boot/grub/i386-pc", "/boot/grub2/i386-pc" };
@@ -295,7 +296,7 @@ static BOOL check_iso_props(const char* psz_dirname, int64_t file_length, const
 		}
 
 		// Split a >4GB install.wim if the target filesystem is FAT
-		if (file_length >= 4 * GB && psz_dirname != NULL && IS_FAT(fs_type) && img_report.has_4GB_file == 0x81) {
+		if (file_length >= 4 * GB && psz_dirname != NULL && IS_FAT(fs_type) && img_report.has_4GB_file == 0x11) {
 			if (safe_stricmp(&psz_dirname[max(0, ((int)safe_strlen(psz_dirname)) -
 				((int)strlen(sources_str)))], sources_str) == 0) {
 				char wim_path[4 * MAX_PATH];
@@ -427,7 +428,7 @@ static BOOL check_iso_props(const char* psz_dirname, int64_t file_length, const
 								"?:%s", psz_fullpath);
 							img_report.wininst_index++;
 							if (file_length >= 4 * GB)
-								img_report.has_4GB_file |= 0x80;
+								img_report.has_4GB_file |= (0x10 << i);
 						}
 					}
 				}
@@ -457,7 +458,7 @@ static BOOL check_iso_props(const char* psz_dirname, int64_t file_length, const
 			if (props->is_old_c32[i])
 				img_report.has_old_c32[i] = TRUE;
 		}
-		if (file_length >= 4 * GB)
+		if (file_length >= 4 * GB && (img_report.has_4GB_file & 0x0f) != 0x0f)
 			img_report.has_4GB_file++;
 		// Compute projected size needed (NB: ISO_BLOCKSIZE = UDF_BLOCKSIZE)
 		if (file_length != 0)
@@ -738,7 +739,9 @@ static int udf_extract_files(udf_t *p_udf, udf_dirent_t *p_udf_dirent, const cha
 						hash_write[HASH_MD5](&ctx, buf, buf_size);
 					ISO_BLOCKING(r = WriteFileWithRetry(file_handle, buf, buf_size, &wr_size, WRITE_RETRIES));
 					if (!r || (wr_size != buf_size)) {
-						uprintf("  Error writing file: %s", r ? "Short write detected" : WindowsErrorString());
+						if (r)
+							SetLastError(ERROR_WRITE_FAULT);
+						uprintf("  Error writing file: %s", WindowsErrorString());
 						goto out;
 					}
 					file_length -= wr_size;
@@ -775,6 +778,8 @@ static int udf_extract_files(udf_t *p_udf, udf_dirent_t *p_udf_dirent, const cha
 	return 0;
 
 out:
+	if (GetLastError() != ERROR_SUCCESS)
+		ErrorStatus = RUFUS_ERROR(GetLastError());
 	udf_dirent_free(p_udf_dirent);
 	ISO_BLOCKING(safe_closehandle(file_handle));
 	safe_free(psz_sanpath);
@@ -793,7 +798,7 @@ static int iso_extract_files(iso9660_t* p_iso, const char *psz_path)
 	BOOL is_symlink, is_identical, create_file, free_p_statbuf = FALSE;
 	int length, r = 1;
 	char psz_fullpath[MAX_PATH], *psz_basename = NULL, *psz_sanpath = NULL;
-	char tmp[128], target_path[256];
+	char tmp[128], target_path[256], *last_slash;
 	const char *psz_iso_name = &psz_fullpath[strlen(psz_extract_dir)];
 	_Static_assert(ISO_BUFFER_SIZE % ISO_BLOCKSIZE == 0,
 		"ISO_BUFFER_SIZE is not a multiple of ISO_BLOCKSIZE");
@@ -969,6 +974,21 @@ static int iso_extract_files(iso9660_t* p_iso, const char *psz_path)
 			if (create_file) {
 				file_handle = CreatePreallocatedFile(psz_sanpath, GENERIC_READ | GENERIC_WRITE,
 					FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, file_length);
+				if (file_handle == INVALID_HANDLE_VALUE && GetLastError() == ERROR_PATH_NOT_FOUND) {
+					// Some folks (umbrelos) managed to master their ISOs in a manner where some
+					// directories don't exist (or don't have _STAT_DIR) but still have files,
+					// in which case our approach, that expects a sane layout with directories
+					// properly declared before the files they contain, breaks. Therefore:
+					last_slash = strrchr(psz_sanpath, '/');
+					if (last_slash != NULL) {
+						*last_slash = '\0';
+						uprintf("WARNING: Directory '%s/' was improperly mastered on the source image!", &psz_sanpath[2]);
+						_mkdirExU(psz_sanpath);
+						*last_slash = '/';
+						file_handle = CreatePreallocatedFile(psz_sanpath, GENERIC_READ | GENERIC_WRITE,
+							FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, file_length);
+					}
+				}
 				if (file_handle == INVALID_HANDLE_VALUE) {
 					err = GetLastError();
 					uprintf("  Unable to create file: %s", WindowsErrorString());
@@ -1003,7 +1023,9 @@ static int iso_extract_files(iso9660_t* p_iso, const char *psz_path)
 							hash_write[HASH_MD5](&ctx, buf, buf_size);
 						ISO_BLOCKING(r = WriteFileWithRetry(file_handle, buf, buf_size, &wr_size, WRITE_RETRIES));
 						if (!r || wr_size != buf_size) {
-							uprintf("  Error writing file: %s", r ? "Short write detected" : WindowsErrorString());
+							if (r)
+								SetLastError(ERROR_WRITE_FAULT);
+							uprintf("  Error writing file: %s", WindowsErrorString());
 							goto out;
 						}
 						file_length -= wr_size;
@@ -1038,6 +1060,8 @@ static int iso_extract_files(iso9660_t* p_iso, const char *psz_path)
 	r = 0;
 
 out:
+	if (r != 0 && GetLastError() != ERROR_SUCCESS)
+		ErrorStatus = RUFUS_ERROR(GetLastError());
 	ISO_BLOCKING(safe_closehandle(file_handle));
 	if (p_entlist != NULL)
 		iso9660_filelist_free(p_entlist);
@@ -1317,7 +1341,9 @@ out:
 		for (k = (int)safe_strlen(img_report.label) - 1; ((k > 0) && (isspaceU(img_report.label[k]))); k--)
 			img_report.label[k] = 0;
 		// We use the fact that UDF_BLOCKSIZE and ISO_BLOCKSIZE are the same here
-		img_report.projected_size = total_blocks * ISO_BLOCKSIZE;
+		// Also, we add 1% extra requirement, on account that we most likely use a 4k or higher cluster size
+		// whereas ISO_BLOCKSIZE is 2k, which means we'll need extra spaces if there are many small files.
+		img_report.projected_size = (uint64_t)((double)total_blocks * ISO_BLOCKSIZE * 1.01f);
 		// We will link the existing isolinux.cfg from a syslinux.cfg we create
 		// If multiple config files exist, choose the one with the shortest path
 		// (so that a '/syslinux.cfg' is preferred over a '/isolinux/isolinux.cfg')
@@ -2172,20 +2198,44 @@ void OpticalDiscSaveImage(void)
 // Create an ISO image from the currently selected drive, using oscdimg.exe
 DWORD WINAPI IsoSaveImageThread(void* param)
 {
-	DWORD r;
+	DWORD r = ERROR_NOT_FOUND;
+	HANDLE exe = INVALID_HANDLE_VALUE;
 	IMG_SAVE* img_save = (IMG_SAVE*)param;
-	char cmd[2 * MAX_PATH], letters[27], * label;
+	char cmd[2 * KB], letters[27], *label;
 
 	if (!GetDriveLabel(SelectedDrive.DeviceNumber, letters, &label, TRUE) || letters[0] == '\0')
-		ExitThread(ERROR_NOT_FOUND);
+		goto out;
+
+	// Get a lock and validate that the oscdimg.exe has not been tampered with
+	static_sprintf(cmd, "%s\\%s\\oscdimg_%s.exe", app_data_dir, FILES_DIR, APPLICATION_ARCH);
+	exe = CreateFileU(cmd, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+	if (exe == INVALID_HANDLE_VALUE) {
+		uprintf("ERROR: Could not lock 'oscdimg.exe'");
+		r = GetLastError();
+		goto out;
+	}
+	if (!FileMatchesHash(cmd, OSCDIMG_HASH)) {
+		uprintf("ERROR: Existing 'oscdimg.exe' hash does not match expected value!");
+		r = ERROR_INVALID_IMAGE_HASH;
+		goto out;
+	}
+
+	// UDF labels cannot be more than 32 characters (and if we have more than 32 chars we are
+	// using the static modifiable char buffer from GetDriveLabel(), so we can alter it).
+	if (strlen(label) > 32)
+		label[32] = '\0';
+
 	// Save to UDF only, as Microsoft's implementation of ISO-9660 doesn't support multiextent
 	// and produces BROKEN images if you try to add files larger than 4 GB.
 	// Plus ISO-9660/Joliet limits labels to 16 characters and has issues with long paths.
-	static_sprintf(cmd, "%s\\%s\\oscdimg.exe -g -h -k -l\"%s\" -m -u2 -udfver102 %c:\\ %s",
-		app_data_dir, FILES_DIR, label, letters[0], img_save->ImagePath);
+	static_sprintf(cmd, "\"%s\\%s\\oscdimg_%s.exe\" -g -h -k -l\"%s\" -m -u2 -udfver102 %c:\\ \"%s\"",
+		app_data_dir, FILES_DIR, APPLICATION_ARCH, label, letters[0], img_save->ImagePath);
 	uprintf("Running command: '%s'", cmd);
 	// For detecting typical oscdimg commandline progress report of type: "\r15.5% complete"
 	r = RunCommandWithProgress(cmd, sysnative_dir, FALSE, MSG_261, ".*\r([0-9\\.]+)% complete.*");
+
+out:
+	safe_closehandle(exe);
 	if (r != 0 && !IS_ERROR(ErrorStatus)) {
 		SetLastError(r);
 		uprintf("Failed to write ISO image: %s", WindowsErrorString());
