@@ -582,13 +582,13 @@ HANDLE CreateFileWithTimeout(LPCSTR lpFileName, DWORD dwDesiredAccess, DWORD dwS
 			CancelSynchronousIo(hThread);
 			switch (WaitForSingleObject(hThread, 30000)) {
 			case WAIT_TIMEOUT:
-				uprintf("File was not created within timeout duration");
+				uprintf("Could not open file or device within timeout duration");
 				break;
 			case WAIT_OBJECT_0:
-				uprintf("File creation aborted by user");
+				uprintf("Operation aborted by user");
 				break;
 			default:
-				uprintf("Error while waiting for file to ne created: %s", WindowsErrorString());
+				uprintf("Error while waiting for file or device to be opened: %s", WindowsErrorString());
 				break;
 			}
 			params.dwError = WAIT_TIMEOUT;
@@ -641,13 +641,14 @@ BOOL WriteFileWithRetry(HANDLE hFile, LPCVOID lpBuffer, DWORD nNumberOfBytesToWr
 		} else {
 			uprintf("Write error %s", WindowsErrorString());
 			LastWriteError = RUFUS_ERROR(GetLastError());
+			if (LastWriteError == RUFUS_ERROR(ERROR_DISK_FULL))
+				break;
 		}
 		// If we can't reposition for the next run, just abort
 		if (!readFilePointer)
 			break;
 		if (nTry < nNumRetries) {
 			uprintf("Retrying in %d seconds...", WRITE_TIMEOUT / 1000);
-			// TODO: Call GetProcessSearch() here?
 			Sleep(WRITE_TIMEOUT);
 		}
 	}
